#include <gdk/gdkkeysyms.h>
#include "gui.h"

#define EVENT_METHOD(i, x) GTK_WIDGET_GET_CLASS(i)->x

void main_window_init();
void time_window_init();
void display_window_init();
void stat_window_init();
void main_window_accel_init();
void about_window_init();

gboolean
timer_event(gpointer data) // {{{
{
	Snoop *s = global_snoop;
	switch (s->timer_state) {
		case TIMER_RUNNING:
			break;
		case TIMER_STOP:
			return FALSE;
		case TIMER_RESTART:
			s->timer_state = TIMER_RUNNING;
			g_timeout_add(s->refresh_interval, timer_event, NULL);
			return FALSE;
		default:
			return FALSE;
	}
	update();
	return TRUE;
} // }}}

void
gui_init() // {{{
{
	Snoop *s = global_snoop;

	main_window_init();

	// set the icon
    	GtkIconTheme *icon_theme = gtk_icon_theme_get_default();
	if (icon_theme) {
	    GError *error = NULL;
	    GdkPixbuf *icon = gtk_icon_theme_load_icon(icon_theme, "eog", 32, 0, &error);
	    if (icon) {
			gtk_window_set_icon(GTK_WINDOW(s->main_window), icon);
			g_object_unref(icon);
	    } else {
			fprintf(stderr, "%s\n", error->message);
			g_error_free(error);
	    }
	    // g_object_unref(icon_theme);
	}
	gtk_widget_show_all(s->main_window);
	gtk_window_set_keep_above((GtkWindow*)s->main_window, 
		s->on_top == ENABLE ? TRUE: FALSE);

	update_title();

	if (s->ruler == DISABLE) {
		gtk_check_menu_item_set_active(
			(GtkCheckMenuItem*)(s->item_ruler), FALSE);
		unput_ruler();
	} else {
		gtk_check_menu_item_set_active(
			(GtkCheckMenuItem*)(s->item_ruler), TRUE);
		put_ruler();
	}

	if (s->grab_mode == MOUSE_ARROW_MODE) {
		s->timer_state = TIMER_RUNNING;
		g_timeout_add(s->refresh_interval, timer_event, NULL);
	}

	time_window_init();
	display_window_init();
	gtk_toggle_button_set_active((GtkToggleButton*)s->item_red, s->red == ENABLE);
	gtk_toggle_button_set_active((GtkToggleButton*)s->item_green, s->green == ENABLE);
	gtk_toggle_button_set_active((GtkToggleButton*)s->item_blue, s->blue == ENABLE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(s->item_nor_histogram), 
			s->nor_histogram == ENABLE ? TRUE : FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(s->item_nor_gradient), 
			s->nor_gradient == ENABLE ? TRUE : FALSE);
	gtk_range_set_value(GTK_RANGE(s->item_gamma), s->gamma);
	gtk_range_set_value(GTK_RANGE(s->item_brightness), s->brightness);
	gtk_range_set_value(GTK_RANGE(s->item_contrast), s->contrast);

	about_window_init();
} // }}}

void
main_window_init() // {{{
{
	// create a new window
	Snoop *snoop = global_snoop;
	GtkWidget *window = snoop->main_window = 
		gtk_window_new (GTK_WINDOW_TOPLEVEL);
	//gtk_widget_set_size_request (GTK_WIDGET (window), 200, 200);
	gtk_window_set_default_size(GTK_WINDOW(window), snoop->width, snoop->height);

	g_signal_connect(G_OBJECT(window), "delete_event",
			G_CALLBACK(gtk_main_quit), NULL);
	g_signal_connect(G_OBJECT(window), "configure_event", 
			G_CALLBACK(handle_configure), NULL);
	g_signal_connect(G_OBJECT(window), "key_press_event", 
			G_CALLBACK(handle_key), NULL);
	g_signal_connect(G_OBJECT(window), "button_press_event", 
			G_CALLBACK(handle_button_press), NULL);
	g_signal_connect(G_OBJECT(window), "button_release_event", 
			G_CALLBACK(handle_button_release), NULL);
	/*
	g_signal_connect(G_OBJECT(window), "motion_notify_event", 
			G_CALLBACK(handle_motion_notify), NULL);
			*/

	GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(window), vbox);

	// XXX stat window must be initialized before menu initialization
	// because a menu item controls its showing/hiding
	stat_window_init();
	
	GtkWidget *menu_bar = menu_init();
	gtk_box_pack_start(GTK_BOX(vbox), menu_bar, FALSE, FALSE, 0);

	// We have to pack GtkImage inside a EventBox to recieve events
	// and clip image.
	GtkWidget *event_box = gtk_event_box_new();
	GtkWidget *image = snoop->image = gtk_image_new();
	/*
	gtk_box_pack_start(GTK_BOX(vbox), event_box, TRUE, TRUE, 0);
	gtk_container_add(GTK_CONTAINER(event_box), image);
	*/

	GtkWidget *table = gtk_table_new(3, 3, FALSE);
	gtk_box_pack_start(GTK_BOX(vbox), table, TRUE, TRUE, 0);

	GtkWidget *hruler = snoop->hruler = gtk_hruler_new();
	GtkWidget *vruler = snoop->vruler = gtk_vruler_new();
	gtk_table_attach (GTK_TABLE (table), hruler, 1, 2, 0, 1,
		GTK_FILL|GTK_EXPAND, GTK_SHRINK, 0, 0);
        gtk_table_attach (GTK_TABLE (table), vruler, 0, 1, 1, 2,
		GTK_SHRINK, GTK_FILL|GTK_EXPAND, 0, 0);

        gtk_table_attach(GTK_TABLE(table), event_box, 1, 2, 1, 2,
		GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);

	gtk_container_add(GTK_CONTAINER(event_box), image);
	gtk_widget_set_size_request (GTK_WIDGET(image), 200, 200);
	
	gtk_widget_add_events(event_box, 
		GDK_POINTER_MOTION_MASK|GDK_POINTER_MOTION_HINT_MASK);
	g_signal_connect(G_OBJECT(event_box), 
		"motion_notify_event", G_CALLBACK(handle_motion_notify), NULL);
	/*
	g_signal_connect_swapped(G_OBJECT(image), "motion_notify_event", 
		G_CALLBACK(EVENT_METHOD(hruler, motion_notify_event)),
		G_OBJECT(hruler));
	g_signal_connect_swapped(G_OBJECT(image), "motion_notify_event", 
		G_CALLBACK(EVENT_METHOD(vruler, motion_notify_event)), 
		G_OBJECT(vruler));
	*/

	global_snoop->statusbar = gtk_statusbar_new();

	PangoFontDescription* font = pango_font_description_new();
	pango_font_description_set_family(font, "monospace");
	pango_font_description_set_size(font, 10*PANGO_SCALE);
	gtk_widget_modify_font(GTK_STATUSBAR(global_snoop->statusbar)->label, font);
	pango_font_description_free(font);

	global_snoop->status_cid = gtk_statusbar_get_context_id(
			(GtkStatusbar*)global_snoop->statusbar, "");
	gtk_statusbar_push((GtkStatusbar*)global_snoop->statusbar, 
			global_snoop->status_cid, "");

	gtk_box_pack_end(GTK_BOX(vbox), global_snoop->statusbar, FALSE, FALSE, 0);

	main_window_accel_init();
} // }}}

void
time_window_init() // {{{
{
	GtkWidget *dialog = global_snoop->time_window = 
		gtk_dialog_new_with_buttons(
			"Setting Refresh Rate",
			(GtkWindow*)global_snoop->main_window, 
			GTK_DIALOG_MODAL, GTK_STOCK_OK, GTK_RESPONSE_OK,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			NULL);
	GtkWidget *label = gtk_label_new("interval (1/1000 second): ");
	GtkWidget *spin = global_snoop->item_refresh_interval = 
		gtk_spin_button_new_with_range(20, INT_MAX, 10);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), global_snoop->refresh_interval);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), label);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), spin);
} // }}}

void
display_window_init() // {{{
{
	Snoop *s = global_snoop;
	GtkWidget *dialog = 
		s->display_window = 
		gtk_dialog_new_with_buttons(
			"Display Options",
			(GtkWindow*)global_snoop->main_window, 
			GTK_DIALOG_MODAL, 
			GTK_STOCK_OK,
			GTK_RESPONSE_OK,
			GTK_STOCK_CANCEL,
			GTK_RESPONSE_CANCEL,
			NULL);

	GtkWidget *box, *box2, *table, *frame;

	// first row: check-box (red, green, blue)
	box = gtk_hbox_new(TRUE, 0);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), box, TRUE, FALSE, 0);

	GtkWidget *red = s->item_red = gtk_check_button_new_with_mnemonic("_Red");
	g_signal_connect(G_OBJECT(red), "toggled",
			G_CALLBACK(handle_check_button), &(s->red));

	GtkWidget *green = s->item_green = gtk_check_button_new_with_mnemonic("_Green");
	g_signal_connect(G_OBJECT(green), "toggled",
			G_CALLBACK(handle_check_button), &(s->green));

	GtkWidget *blue = s->item_blue = gtk_check_button_new_with_mnemonic("_Blue");
	g_signal_connect(G_OBJECT(blue), "toggled",
			G_CALLBACK(handle_check_button), &(s->blue));

	gtk_box_pack_start(GTK_BOX(box), red, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(box), green, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(box), blue, FALSE, FALSE, 0);

	// second row: normal and reference
	// normal: check-box (histogram, gradient)
	box = gtk_hbox_new(TRUE, 0);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
			box, TRUE, FALSE, 0);
	frame = gtk_frame_new("Auto Normalize");
	gtk_box_pack_start(GTK_BOX(box), frame, TRUE, TRUE, 0);
	box2 = gtk_vbox_new(TRUE, 0);
	gtk_container_add(GTK_CONTAINER(frame), box2);

	GtkWidget *nor_histogram = s->item_nor_histogram = 
		gtk_check_button_new_with_mnemonic("_Histogram");
	g_signal_connect(G_OBJECT(nor_histogram), "toggled",
			G_CALLBACK(handle_check_button), 
			&(s->nor_histogram));

	GtkWidget *nor_gradient = s->item_nor_gradient = 
		gtk_check_button_new_with_mnemonic("_Gradient");
	g_signal_connect(G_OBJECT(nor_gradient), "toggled",
			G_CALLBACK(handle_check_button), 
			&(s->nor_gradient));

	gtk_box_pack_start(GTK_BOX(box2), nor_histogram, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(box2), nor_gradient, FALSE, FALSE, 0);

	// reference: spin-button (histogram, gradient)
	frame = gtk_frame_new("reference");
	gtk_box_pack_start(GTK_BOX(box), frame, TRUE, TRUE, 0);
	GtkWidget *l_histogram = gtk_label_new("Histogram");
	GtkWidget *l_gradient = gtk_label_new("Gradient");

	GtkWidget *ref_histogram = s->item_ref_histogram =
		gtk_spin_button_new_with_range(10, 10000, 10);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(ref_histogram), s->ref_histogram);
	g_signal_connect(G_OBJECT(ref_histogram), "value-changed",
			G_CALLBACK(handle_spin_button), 
			&(global_snoop->ref_histogram));

	GtkWidget *ref_gradient = s->item_ref_gradient = 
		gtk_spin_button_new_with_range(10, 255, 1);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(ref_gradient), s->ref_gradient);
	g_signal_connect(G_OBJECT(ref_gradient), "value-changed",
			G_CALLBACK(handle_spin_button), 
			&(global_snoop->ref_gradient));

	table = gtk_table_new(2, 2, FALSE);
	gtk_table_attach((GtkTable*)table, l_histogram, 0, 1, 0, 1, 
			GTK_SHRINK, GTK_EXPAND, 0, 0);
	gtk_table_attach((GtkTable*)table, l_gradient, 0, 1, 1, 2, 
			GTK_SHRINK, GTK_EXPAND, 0, 0);
	gtk_table_attach((GtkTable*)table, ref_histogram, 1, 2, 0, 1, 
			GTK_EXPAND | GTK_FILL, GTK_EXPAND, 0, 0);
	gtk_table_attach((GtkTable*)table, ref_gradient, 1, 2, 1, 2, 
			GTK_EXPAND | GTK_FILL, GTK_EXPAND, 0, 0);
	gtk_container_add(GTK_CONTAINER(frame), table);
	
	// third row: brightness/contrast/gamma
	frame = gtk_frame_new(NULL);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), frame, FALSE, FALSE, 0);
	table = gtk_table_new(3, 2, FALSE);
	gtk_container_add(GTK_CONTAINER(frame), table);
	gtk_table_set_col_spacing((GtkTable*)table, 1, 20);

	// brightness
	GtkWidget *l_brightness = gtk_label_new("Brightness");
	GtkWidget *brightness = s->item_brightness = 
		gtk_hscale_new_with_range(-100, 100, 1);
	//gtk_range_set_value(GTK_RANGE(brightness), 0);
	g_signal_connect(G_OBJECT(brightness), "value-changed",
			G_CALLBACK(handle_brightness), 
			&(global_snoop->brightness));

	gtk_table_attach((GtkTable*)table, l_brightness, 0, 1, 0, 1, 
			GTK_SHRINK, GTK_EXPAND, 0, 0);
	gtk_table_attach((GtkTable*)table, brightness, 1, 2, 0, 1, 
			GTK_EXPAND | GTK_FILL, GTK_EXPAND, 0, 0);

	// contrast
	GtkWidget *l_contrast = gtk_label_new("Contrast");
	GtkWidget *contrast = s->item_contrast = 
		gtk_hscale_new_with_range(-100, 100, 1);
	//gtk_range_set_value(GTK_RANGE(contrast), 0);
	g_signal_connect(G_OBJECT(contrast), "value-changed",
			G_CALLBACK(handle_contrast), 
			&(global_snoop->contrast));

	gtk_table_attach((GtkTable*)table, l_contrast, 0, 1, 1, 2, 
			GTK_SHRINK, GTK_EXPAND, 0, 0);
	gtk_table_attach((GtkTable*)table, contrast, 1, 2, 1, 2, 
			GTK_EXPAND | GTK_FILL, GTK_EXPAND, 0, 0);

	// gamma
	GtkWidget *l_gamma = gtk_label_new("Gamma");
	GtkWidget *gamma = s->item_gamma = 
		gtk_hscale_new_with_range(0, 2, 0.01);
	//gtk_range_set_value(GTK_RANGE(gamma), 1);
	g_signal_connect(G_OBJECT(gamma), "value-changed",
			G_CALLBACK(handle_gamma), 
			&(s->gamma));

	gtk_table_attach((GtkTable*)table, l_gamma, 0, 1, 2, 3, 
			GTK_SHRINK, GTK_EXPAND, 0, 0);
	gtk_table_attach((GtkTable*)table, gamma, 1, 2, 2, 3, 
			GTK_EXPAND | GTK_FILL, GTK_EXPAND, 0, 0);
} // }}}

void
about_window_init() // {{{
{
	GtkWidget *dialog = global_snoop->about_window = 
		gtk_dialog_new_with_buttons(
			"About Snoop",
			(GtkWindow*)global_snoop->main_window, 
			GTK_DIALOG_MODAL, 
			GTK_STOCK_OK, GTK_RESPONSE_OK,
			NULL);
	const char message[] = 
		"Snoop for X\n"
		"Base on Yung-Yu Chuang's snoop\n"
		"Written by \n"
		"Tz-huan Huang (tzhuan@csie.org)\n"
		"Shan-yung Yang (littleshan@cise.org)\n";
	GtkWidget *label = gtk_label_new(message);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), label);

	GtkWidget *frame = gtk_frame_new("Hotkey list");
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), frame);
	const char hotkey[] = 
		"[1-9] zoom factor = n\n"
		"[0] zoom factor = 10\n"
		"[=+] increase zoom factor\n"
		"[-] decrease zoom factor\n"
		"[Esc] quit\n"
		"(other hotkeys list in menu-item)";
	label = gtk_label_new(hotkey);
	gtk_container_add(GTK_CONTAINER(frame), label);
	/*
	GtkWidget *spin = global_snoop->item_refresh_interval = 
		gtk_spin_button_new_with_range(20, INT_MAX, 10);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), global_snoop->refresh_interval);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), spin);
	*/
} // }}}

void
stat_window_init() // {{{
{
	// initialize statistic window
	Snoop* s = global_snoop;
	s->stat_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_size_request(s->stat_window, 260, 200);
	g_signal_connect(G_OBJECT(s->stat_window), "delete_event",
			G_CALLBACK(handle_stat_window_close), NULL);
	GtkWindow* swin = GTK_WINDOW(s->stat_window);
	gtk_window_set_title(swin, "Statistics");
	gtk_window_set_default_size(swin, 260, 200);
	gtk_window_set_resizable(swin, FALSE);
	gtk_window_set_destroy_with_parent(swin, TRUE);
	gtk_window_set_transient_for(swin, GTK_WINDOW(s->main_window));
	gtk_widget_show(s->stat_window);

	GtkBox* vbox = (GtkBox*)gtk_vbox_new(FALSE, 1);
	gtk_container_add(GTK_CONTAINER(s->stat_window), GTK_WIDGET(vbox));
	
	s->stat_image = gtk_image_new();
	gtk_widget_set_size_request(s->stat_image, 256, 200);
	s->item_peak = gtk_label_new("");
	s->item_minmax = gtk_label_new("");
	s->item_avgmed = gtk_label_new("");

	// use a smaller font to show statistics info
	PangoFontDescription* font = pango_font_description_new();
	pango_font_description_set_family(font, "Sans");
	pango_font_description_set_size(font, 10*PANGO_SCALE);
	gtk_widget_modify_font(s->item_peak, font);
	gtk_widget_modify_font(s->item_minmax, font);
	gtk_widget_modify_font(s->item_avgmed, font);
	pango_font_description_free(font);

	gtk_box_pack_start(vbox, s->item_peak, FALSE, TRUE, 2);
	gtk_box_pack_start(vbox, s->stat_image, TRUE, TRUE, 2);
	gtk_box_pack_start(vbox, s->item_minmax, FALSE, TRUE, 2);
	gtk_box_pack_start(vbox, s->item_avgmed, FALSE, TRUE, 2);

	if(s->statistics == ENABLE)
	    gtk_widget_show_all(s->stat_window);
	else
	    gtk_widget_hide(s->stat_window);
} // }}}

void
update_title() // {{{
{
	char msg[256] = "";
	Snoop *s = global_snoop;
	int depth = gdk_drawable_get_depth(
		(GdkDrawable*)gdk_get_default_root_window());
	int screen_width = gdk_screen_width();
	int screen_height = gdk_screen_height();

	sprintf(msg, "snoop x%d [%c] %d-bit (%dx%d)",
		s->zoom_factor, (s->grab_mode == MOUSE_ARROW_MODE ? 'M' : 'D'),
		depth, screen_width, screen_height);
	gtk_window_set_title(GTK_WINDOW(s->main_window), msg);
} // }}}

void
update() // {{{
{
	static GdkPixbuf *bg = NULL;
	Snoop *s = global_snoop;
	Geometry geometry;
	Geometry *g = &geometry;

	prepare_coordinate(&geometry);

	// capture the screenshot
	if (!bg || gdk_pixbuf_get_width(bg) != g->desire_capture_width || 
		gdk_pixbuf_get_height(bg) != g->desire_capture_height) {
		if (bg)
			g_object_unref(bg);
		bg = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, 
			g->desire_capture_width, g->desire_capture_height);
	}

	fill_color(bg, 0, 0, 0, 255);

	GdkPixbuf *src_buffer = gdk_pixbuf_get_from_drawable(bg, 
			(GdkDrawable*)g->root_window, gdk_colormap_get_system(),
			g->capture_left, g->capture_top, 
			g->src_x, g->src_y, 
			g->capture_width, g->capture_height); 
	if (!src_buffer)
		return;

	// update status before filtering
	update_status(src_buffer, g);
	
	// apply filter
	apply_filter(s->filter, src_buffer);

	adjust_color(src_buffer);
	// calculate statistics
	if(s->statistics == ENABLE)
	    update_statistics(src_buffer);

	// scale the screenshot
	//if (scaled_buffer) g_object_unref(tmp);
	GdkPixbuf *scaled_buffer = gdk_pixbuf_scale_simple( src_buffer,
			g->scaled_width, g->scaled_height, GDK_INTERP_NEAREST);

	if (s->grid == ENABLE) draw_grid(scaled_buffer, g->zoom_factor);

	if (s->center_highlight == ENABLE) 
		draw_center(scaled_buffer, g->center_x, g->center_y);

	if (s->highlight == ENABLE){
		switch (s->graph){ 
		    case HORIZONTAL_STEP: 
		    case HORIZONTAL_LINEAR:
			draw_hhighlight(scaled_buffer, g);
			break;

		    case VERTICAL_STEP:
		    case VERTICAL_LINEAR:
			draw_vhighlight(scaled_buffer, g);
			break;
		    
		    default:
			break;
		}
	}

	if (s->graph != NO_GRAPH)
		draw_graph(src_buffer, scaled_buffer, g->center_x, g->center_y);

	// update image 
	GdkPixbuf *tmp = gdk_pixbuf_new_subpixbuf(scaled_buffer, 0, 0,
		g->image_width, g->image_height);
	gtk_image_set_from_pixbuf(GTK_IMAGE(s->image), tmp);

	// unreference GdkPixbuf
	g_object_unref(scaled_buffer);
	g_object_unref(tmp);
} // }}}

void
put_ruler()
{
	//int w, h;
	//gtk_widget_get_size_request(global_snoop->vruler, &w, NULL);
	//gtk_widget_get_size_request(global_snoop->hruler, NULL, &h);
	gtk_widget_show(global_snoop->hruler);
	gtk_widget_show(global_snoop->vruler);
	update_ruler();
}

void
unput_ruler()
{
	gtk_widget_hide(global_snoop->hruler);
	gtk_widget_hide(global_snoop->vruler);
}

void
update_ruler()
{
	Snoop *s = global_snoop;
	int width = s->image->allocation.width;
	int height = s->image->allocation.height;
	//gtk_widget_get_size_request(s->image, &width, &height);
	double hupper = (double)width/s->zoom_factor;
	double vupper = (double)height/s->zoom_factor;
	gtk_ruler_set_range((GtkRuler*)s->hruler, 0, hupper, 0, hupper);
	gtk_ruler_set_range((GtkRuler*)s->vruler, 0, vupper, 0, vupper);
}

void
main_window_accel_init()
{
	Snoop *s = global_snoop;
	GtkAccelGroup *a = s->accel_group = gtk_accel_group_new();
        gtk_window_add_accel_group(GTK_WINDOW(s->main_window), a);
	gtk_widget_add_accelerator(s->item_on_top, "activate", a, GDK_F2, 0, GTK_ACCEL_VISIBLE); 
	gtk_widget_add_accelerator(s->item_grid, "activate", a, GDK_F3, 0, GTK_ACCEL_VISIBLE); 
	//gtk_widget_add_accelerator(s->item_hex_mode, "activate", a, GDK_F3, 0, GTK_ACCEL_VISIBLE); 
	//gtk_widget_add_accelerator(s->item_inverse_y, "activate", a, GDK_F3, 0, GTK_ACCEL_VISIBLE); 
	//gtk_widget_add_accelerator(s->item_center_highlight, "activate", a, GDK_F3, 0, GTK_ACCEL_VISIBLE); 
	//gtk_widget_add_accelerator(s->item_highlight, "activate", a, GDK_F3, 0, GTK_ACCEL_VISIBLE); 
	gtk_widget_add_accelerator(s->item_ruler, "activate", a, GDK_r, 0, GTK_ACCEL_VISIBLE); 
	//gtk_widget_add_accelerator(s->item_filter[], "activate", a, GDK_F3, 0, GTK_ACCEL_VISIBLE); 
	gtk_widget_add_accelerator(s->item_grab_mode[MOUSE_ARROW_MODE], "activate", a, GDK_m, 0, GTK_ACCEL_VISIBLE); 
	gtk_widget_add_accelerator(s->item_grab_mode[DRAG_MODE], "activate", a, GDK_a, 0, GTK_ACCEL_VISIBLE); 
	gtk_widget_add_accelerator(s->item_graph[NO_GRAPH], "activate", a, GDK_o, 0, GTK_ACCEL_VISIBLE); 
	gtk_widget_add_accelerator(s->item_graph[HORIZONTAL_STEP], "activate", a, GDK_H, GDK_SHIFT_MASK, GTK_ACCEL_VISIBLE); 
	gtk_widget_add_accelerator(s->item_graph[HORIZONTAL_LINEAR], "activate", a, GDK_h, 0, GTK_ACCEL_VISIBLE); 
	gtk_widget_add_accelerator(s->item_graph[VERTICAL_STEP], "activate", a, GDK_V, GDK_SHIFT_MASK, GTK_ACCEL_VISIBLE); 
	gtk_widget_add_accelerator(s->item_graph[VERTICAL_LINEAR], "activate", a, GDK_v, 0, GTK_ACCEL_VISIBLE); 
	gtk_widget_add_accelerator(s->item_statistics, "activate", a, GDK_F4, 0, GTK_ACCEL_VISIBLE); 
	gtk_widget_add_accelerator(s->item_quit, "activate", a, GDK_Escape, 0, GTK_ACCEL_VISIBLE); 
	gtk_widget_add_accelerator(s->item_update, "activate", a, GDK_u, 0, GTK_ACCEL_VISIBLE); 
}
