#include "handler.h"

void
handle_quit(GtkCheckMenuItem *item, gpointer data)
{
	gtk_main_quit ();
}

void
handle_update(GtkCheckMenuItem *item, gpointer data)
{
	update();
}

void
handle_about(GtkCheckMenuItem *item, gpointer data)
{
	gtk_widget_show_all(global_snoop->about_window);
	gtk_dialog_run((GtkDialog*)global_snoop->about_window);
	gtk_widget_hide_all(global_snoop->about_window);
}

void
handle_option(GtkCheckMenuItem *item, gpointer data) // {{{
{
	*(Option*)data = 
		gtk_check_menu_item_get_active(item) ? ENABLE : DISABLE;
} // }}}

void
handle_check_button(GtkToggleButton *button, gpointer data) // {{{
{
	*(Option*)data = 
		gtk_toggle_button_get_active(button) ? ENABLE : DISABLE;
} // }}}

void 
handle_spin_button(GtkSpinButton *spin, gpointer data) // {{{
{
	*(int*)data = gtk_spin_button_get_value_as_int(spin);
} // }}}

void
handle_brightness(GtkRange *range, gpointer data) // {{{
{
	
	set_brightnessLUT(*(int*)data = gtk_range_get_value(range));
} // }}}

void
handle_contrast(GtkRange *range, gpointer data) // {{{
{
	
	set_contrastLUT(*(int*)data = gtk_range_get_value(range));
} // }}}

void
handle_gamma(GtkRange *range, gpointer data) // {{{
{
	set_gammaLUT(*(double*)data = gtk_range_get_value(range));
} // }}}

void
handle_ruler(GtkCheckMenuItem *item, gpointer data) // {{{
{
	*(Option*)data = 
		gtk_check_menu_item_get_active(item) ? ENABLE : DISABLE;
	
	if (*(Option*)data == ENABLE)
		put_ruler();
	else
		unput_ruler();
} // }}}

void
handle_filter(GtkRadioMenuItem *item, gpointer data) // {{{
{
	global_snoop->filter = (Filter)data;
} // }}}

void
handle_colorspace(GtkRadioMenuItem *item, gpointer data) // {{{
{
	global_snoop->colorspace = (Colorspace)data;
} // }}}

void
handle_refresh_rate_dialog(GtkMenuItem *menuitem, gpointer spin) // {{{
{
	gtk_widget_show_all(global_snoop->time_window);
	gint result = gtk_dialog_run((GtkDialog*)global_snoop->time_window);
	Snoop *s = global_snoop;
	if (result == GTK_RESPONSE_OK) {
		gint interval = gtk_spin_button_get_value_as_int
			(GTK_SPIN_BUTTON(s->item_refresh_interval));
		if(interval >= 20 && interval < INT_MAX){
			s->refresh_interval = interval;
			s->timer_state = TIMER_RESTART;
		}
	} else {
		gtk_spin_button_set_value(
			GTK_SPIN_BUTTON(s->item_refresh_interval), s->refresh_interval);
	}
	gtk_widget_hide_all(global_snoop->time_window);
} // }}}

void
handle_display_option_dialog(GtkMenuItem *menuitem, gpointer data) // {{{
{
	Snoop *s = global_snoop;
	// save the current display options
	Option red = s->red;
	Option green = s->green;
	Option blue = s->blue;
	Option nor_histogram = s->nor_histogram;
	Option nor_gradient = s->nor_gradient;
	int ref_histogram = s->ref_histogram;
	int ref_gradient = s->ref_gradient;
	int brightness = s->brightness;
	int contrast = s->contrast;
	double gamma = s->gamma;

	gtk_widget_show_all(global_snoop->display_window);
	gint result = gtk_dialog_run((GtkDialog*)global_snoop->display_window);
	if (result == GTK_RESPONSE_CANCEL) {
		gtk_toggle_button_set_active((GtkToggleButton*)s->item_red, red == ENABLE);
		gtk_toggle_button_set_active((GtkToggleButton*)s->item_green, green == ENABLE);
		gtk_toggle_button_set_active((GtkToggleButton*)s->item_blue, blue == ENABLE);
		gtk_toggle_button_set_active((GtkToggleButton*)s->item_nor_histogram, nor_histogram == ENABLE);
		gtk_toggle_button_set_active((GtkToggleButton*)s->item_nor_gradient, nor_gradient == ENABLE);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(s->item_ref_histogram), ref_histogram);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(s->item_ref_gradient), ref_gradient);
		gtk_range_set_value(GTK_RANGE(s->item_brightness), brightness);
		gtk_range_set_value(GTK_RANGE(s->item_contrast), contrast);
		gtk_range_set_value(GTK_RANGE(s->item_gamma), gamma);
	}
	gtk_widget_hide_all(global_snoop->display_window);
} // }}}

void
handle_graph(GtkRadioMenuItem *item, gpointer data) // {{{
{
	global_snoop->graph = (Graph)data;
} // }}}

void
handle_grab_mode(GtkRadioMenuItem *item, gpointer data) // {{{
{
	global_snoop->grab_mode = (GrabMode)data;
	update_title();
	if (global_snoop->grab_mode == DRAG_MODE)
		global_snoop->timer_state = TIMER_STOP;	
	else if (global_snoop->grab_mode == MOUSE_ARROW_MODE) {
		global_snoop->timer_state = TIMER_RUNNING;
		g_timeout_add(global_snoop->refresh_interval, timer_event, NULL);
	}
} // }}}

void 
handle_saveconfig(GtkMenuItem *item, gpointer data)
{
	write_config(NULL);
}

void
handle_ontop(GtkCheckMenuItem* item, gpointer data)
{
	global_snoop->on_top = 
		gtk_check_menu_item_get_active(item) ? ENABLE : DISABLE;
	gtk_window_set_keep_above((GtkWindow*)global_snoop->main_window, 
		global_snoop->on_top == ENABLE ? TRUE: FALSE);
}

gint
handle_configure(GtkWindow* window, gpointer data)
{
	gtk_window_get_size(window, &(global_snoop->width), &(global_snoop->height));
	update_ruler();
	return FALSE;
}


void handle_statistics(GtkCheckMenuItem* item, gpointer data){
    Snoop* s = global_snoop;
    // toggle statistics window on/off
    s->statistics = gtk_check_menu_item_get_active(item) ? ENABLE : DISABLE;
    if(s->statistics == ENABLE)
	gtk_widget_show_all(s->stat_window);
    else
	gtk_widget_hide_all(s->stat_window);
}

gint handle_stat_window_close(GtkWindow* window, gpointer data){
    gtk_check_menu_item_set_active(
	    GTK_CHECK_MENU_ITEM(global_snoop->item_statistics), FALSE);
    return TRUE;
}

void handle_copy(GtkMenuItem* item, gpointer data){
    // copy the image into the clipboard
    //GtkClipboard* clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
    //GdkPixbuf* pixbuf = gtk_image_get_pixbuf(GTK_IMAGE(global_snoop->image));
    // FIXME gtk_clipboard_set_image requires Gtk-2.6!
    //gtk_clipboard_set_image(clipboard, pixbuf);
}
