#ifndef __SNOOP_H__
#define __SNOOP_H__

#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>

typedef enum { TIMER_RUNNING, TIMER_STOP, TIMER_RESTART } TimerState;
typedef enum { DISABLE, ENABLE } Option;
typedef enum { RGB, RGB_HEX, HSV } Colorspace;
typedef enum { NO_FILTER, GRADIENT } Filter;
typedef enum { MOUSE_ARROW_MODE, DRAG_MODE} GrabMode;

typedef enum {
	NO_GRAPH, HORIZONTAL_STEP, HORIZONTAL_LINEAR, 
	VERTICAL_STEP, VERTICAL_LINEAR
} Graph;

typedef struct { // {{{
	// option variable 
	int zoom_factor;

	Option autosave;
	Option on_top;
	Option update;
	Option grid;
	Option ruler;
	Option hex_mode;
	Option inverse_y;
	// a red square in the center
	Option center_highlight;
	Option red, green, blue;
	Option nor_histogram, nor_gradient;
	int ref_histogram, ref_gradient;
	int brightness, contrast;
	double gamma;
	unsigned char brightnessLUT[256], contrastLUT[256], gammaLUT[256];


	// a half-transparent band in the center when display graph
	Option highlight;
	// statistics window
	Option statistics;
	Colorspace colorspace;
	Filter filter;
	Graph graph;
	GrabMode grab_mode;

	int refresh_interval;	// in msecond (0.001 second)
	int width;
	int height;
	TimerState timer_state;

	GtkWidget *main_window;
	GtkWidget *time_window;
	GtkWidget *display_window;
	GtkWidget *stat_window;
	GtkWidget *about_window;

	GtkWidget *image, *stat_image;
	GtkWidget *statusbar;
	GtkWidget *hruler, *vruler;
	guint status_cid;
	guint status_mid;

	GtkWidget *item_refresh_interval;
	GtkWidget *item_red, *item_green, *item_blue;
	GtkWidget *item_nor_histogram, *item_nor_gradient;
	GtkWidget *item_ref_histogram, *item_ref_gradient;
	GtkWidget *item_brightness, *item_contrast, *item_gamma;
	// labels in statistics window
	GtkWidget *item_peak, *item_minmax, *item_avgmed;

	GtkWidget *item_on_top;
	GtkWidget *item_grid;
	GtkWidget *item_hex_mode;
	GtkWidget *item_inverse_y;
	GtkWidget *item_center_highlight;
	GtkWidget *item_highlight;
	GtkWidget *item_ruler;
	GtkWidget **item_colorspace;
	GtkWidget **item_filter;
	GtkWidget **item_grab_mode;
	GtkWidget **item_graph;
	GtkWidget *item_statistics;

	GtkWidget *item_quit;
	GtkWidget *item_update;

	GtkAccelGroup *accel_group;
} Snoop; // }}}

Snoop *snoop_new(void);
// set all options to default values
void snoop_set_default(Snoop* s);
void snoop_destroy(Snoop*);

// read/write config file
int read_config(const char*);
void write_config(const char*);

extern Snoop *global_snoop;

#endif
